var Hash = require("bn-lang").Hash;
var StringBuilder = require("bn-lang-util").StringBuilder;

/**
* @class Node
*/

var Node = function(){this.init.apply(this, arguments);};
exports.Node = Node;

Node.prototype.init = function(_tag) {

	this.tag = _tag;
	this.attributes = new Hash();
	this.children = [];	
};

Node.prototype.setAttributes = function(_attrs) {

	for (var _key in _attrs) {

		var mappedKey = _key == 'cls' ? 'class' : _key;

		this.attributes.put(mappedKey, _attrs[_key]);
	}
};

Node.prototype.addChild = function(_childText) {

	this.children.add(_childText);
};

Node.prototype.render = function(_lpad) {

	var lpad = _lpad || 0;

	var output = new StringBuilder();

	output.append(Node.getPadding(lpad));

	output.append("<" + this.tag);

	this.attributes.each(function(_value, _key){

		output.append(" " + _key + "=\"" + _value + "\"");
	});

	if(Node.isSelfClosingTag(this.tag)) {

		output.append(" />\n");

	} else {

		output.append(">");

		var multiline = Node.isMultiLineTag(this.children);

		if (multiline) { output.append("\n"); }

		this.children.each(function(_child){

			output.append(_child.render(lpad + 2));
		});

		if (multiline) output.append(Node.getPadding(lpad));

		output.append("</" + this.tag + ">\n");
	}

	return output.toString();
};

Node.getPadding = function(_amount) {

	return new Array(_amount + 1).join(" ");
};

Node.isMultiLineTag = function(_childern) {

	var childSize = _childern.size();
	var isMultiLine = childSize > 0;

	if (childSize == 1 && _childern.get(0).constructor == TextNode) isMultiLine = false;

	return isMultiLine;
};

Node.isSelfClosingTag = function(_tag) {

	return Node.SELF_CLOSING_TAGS.include(_tag);
};

Node.SELF_CLOSING_TAGS = [];
Node.setSelfClosingTags = function(_tags) {

	Node.SELF_CLOSING_TAGS = _tags;
};
Node.HTML_SELF_CLOSING_TAGS = ["img", "meta", "br", "hr", "input"];
